@@ -63,17 +63,18 @@ BaseEstimator.__repr__ for pretty-printing estimators"""
 # - long sequences (lists, tuples, dict items) with more than N elements are
 #   shortened using ellipsis (', ...') at the end.
 
-from inspect import signature
+import inspect
 import pprint
 from collections import OrderedDict
 
-from ..base import BaseEstimator
 from .._config import get_config
+from ..base import BaseEstimator
 from . import is_scalar_nan
 
 
 class KeyValTuple(tuple):
     """Dummy class for correctly rendering key-value tuples from dicts."""
+
     def __repr__(self):
         # needed for _dispatch[tuple.__repr__] not to be overridden
         return super().__repr__()
@@ -81,6 +82,7 @@ class KeyValTuple(tuple):
 
 class KeyValTupleParam(KeyValTuple):
     """Dummy class for correctly rendering key-value tuples from parameters."""
+
     pass
 
 
@@ -89,16 +91,26 @@ def _changed_params(estimator):
     estimator with non-default values."""
 
     params = estimator.get_params(deep=False)
-    filtered_params = {}
-    init_func = getattr(estimator.__init__, 'deprecated_original',
-                        estimator.__init__)
-    init_params = signature(init_func).parameters
+    init_func = getattr(estimator.__init__, "deprecated_original", estimator.__init__)
+    init_params = inspect.signature(init_func).parameters
     init_params = {name: param.default for name, param in init_params.items()}
-    for k, v in params.items():
-        if (repr(v) != repr(init_params[k]) and
-                not (is_scalar_nan(init_params[k]) and is_scalar_nan(v))):
-            filtered_params[k] = v
-    return filtered_params
+
+    def has_changed(k, v):
+        if k not in init_params:  # happens if k is part of a **kwargs
+            return True
+        if init_params[k] == inspect._empty:  # k has no default value
+            return True
+        # try to avoid calling repr on nested estimators
+        if isinstance(v, BaseEstimator) and v.__class__ != init_params[k].__class__:
+            return True
+        # Use repr as a last resort. It may be expensive.
+        if repr(v) != repr(init_params[k]) and not (
+            is_scalar_nan(init_params[k]) and is_scalar_nan(v)
+        ):
+            return True
+        return False
+
+    return {k: v for k, v in params.items() if has_changed(k, v)}
 
 
 class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
@@ -117,7 +129,7 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
       here)
     - format() directly calls _safe_repr() for a first try at rendering the
       object
-    - _safe_repr formats the whole object reccursively, only calling itself,
+    - _safe_repr formats the whole object recursively, only calling itself,
       not caring about line length or anything
     - back to _format(), if the output string is too long, _format() then calls
       the appropriate _pprint_TYPE() method (e.g. _pprint_list()) depending on
@@ -152,26 +164,34 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
     KeyValTupleParam for this.
     """
 
-    def __init__(self, indent=1, width=80, depth=None, stream=None, *,
-                 compact=False, indent_at_name=True,
-                 n_max_elements_to_show=None):
+    def __init__(
+        self,
+        indent=1,
+        width=80,
+        depth=None,
+        stream=None,
+        *,
+        compact=False,
+        indent_at_name=True,
+        n_max_elements_to_show=None,
+    ):
         super().__init__(indent, width, depth, stream, compact=compact)
         self._indent_at_name = indent_at_name
         if self._indent_at_name:
             self._indent_per_level = 1  # ignore indent param
-        self._changed_only = get_config()['print_changed_only']
+        self._changed_only = get_config()["print_changed_only"]
         # Max number of elements in a list, dict, tuple until we start using
         # ellipsis. This also affects the number of arguments of an estimators
         # (they are treated as dicts)
         self.n_max_elements_to_show = n_max_elements_to_show
 
     def format(self, object, context, maxlevels, level):
-        return _safe_repr(object, context, maxlevels, level,
-                          changed_only=self._changed_only)
+        return _safe_repr(
+            object, context, maxlevels, level, changed_only=self._changed_only
+        )
 
-    def _pprint_estimator(self, object, stream, indent, allowance, context,
-                          level):
-        stream.write(object.__class__.__name__ + '(')
+    def _pprint_estimator(self, object, stream, indent, allowance, context, level):
+        stream.write(object.__class__.__name__ + "(")
         if self._indent_at_name:
             indent += len(object.__class__.__name__)
 
@@ -180,24 +200,26 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
         else:
             params = object.get_params(deep=False)
 
-        params = OrderedDict((name, val)
-                             for (name, val) in sorted(params.items()))
+        params = OrderedDict((name, val) for (name, val) in sorted(params.items()))
 
-        self._format_params(params.items(), stream, indent, allowance + 1,
-                            context, level)
-        stream.write(')')
+        self._format_params(
+            params.items(), stream, indent, allowance + 1, context, level
+        )
+        stream.write(")")
 
-    def _format_dict_items(self, items, stream, indent, allowance, context,
-                           level):
+    def _format_dict_items(self, items, stream, indent, allowance, context, level):
         return self._format_params_or_dict_items(
-            items, stream, indent, allowance, context, level, is_dict=True)
+            items, stream, indent, allowance, context, level, is_dict=True
+        )
 
     def _format_params(self, items, stream, indent, allowance, context, level):
         return self._format_params_or_dict_items(
-            items, stream, indent, allowance, context, level, is_dict=False)
+            items, stream, indent, allowance, context, level, is_dict=False
+        )
 
-    def _format_params_or_dict_items(self, object, stream, indent, allowance,
-                                     context, level, is_dict):
+    def _format_params_or_dict_items(
+        self, object, stream, indent, allowance, context, level, is_dict
+    ):
         """Format dict items or parameters respecting the compact=True
         parameter. For some reason, the builtin rendering of dict items doesn't
         respect compact=True and will use one line per key-value if all cannot
@@ -210,8 +232,8 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
         """
         write = stream.write
         indent += self._indent_per_level
-        delimnl = ',\n' + ' ' * indent
-        delim = ''
+        delimnl = ",\n" + " " * indent
+        delim = ""
         width = max_width = self._width - indent + 1
         it = iter(object)
         try:
@@ -222,7 +244,7 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
         n_items = 0
         while not last:
             if n_items == self.n_max_elements_to_show:
-                write(', ...')
+                write(", ...")
                 break
             n_items += 1
             ent = next_ent
@@ -238,7 +260,7 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
                 vrepr = self._repr(v, context, level)
                 if not is_dict:
                     krepr = krepr.strip("'")
-                middle = ': ' if is_dict else '='
+                middle = ": " if is_dict else "="
                 rep = krepr + middle + vrepr
                 w = len(rep) + 2
                 if width < w:
@@ -248,14 +270,15 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
                 if width >= w:
                     width -= w
                     write(delim)
-                    delim = ', '
+                    delim = ", "
                     write(rep)
                     continue
             write(delim)
             delim = delimnl
             class_ = KeyValTuple if is_dict else KeyValTupleParam
-            self._format(class_(ent), stream, indent,
-                         allowance if last else 1, context, level)
+            self._format(
+                class_(ent), stream, indent, allowance if last else 1, context, level
+            )
 
     def _format_items(self, items, stream, indent, allowance, context, level):
         """Format the items of an iterable (list, tuple...). Same as the
@@ -265,9 +288,9 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
         write = stream.write
         indent += self._indent_per_level
         if self._indent_per_level > 1:
-            write((self._indent_per_level - 1) * ' ')
-        delimnl = ',\n' + ' ' * indent
-        delim = ''
+            write((self._indent_per_level - 1) * " ")
+        delimnl = ",\n" + " " * indent
+        delim = ""
         width = max_width = self._width - indent + 1
         it = iter(items)
         try:
@@ -278,7 +301,7 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
         n_items = 0
         while not last:
             if n_items == self.n_max_elements_to_show:
-                write(', ...')
+                write(", ...")
                 break
             n_items += 1
             ent = next_ent
@@ -298,33 +321,33 @@ class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
                 if width >= w:
                     width -= w
                     write(delim)
-                    delim = ', '
+                    delim = ", "
                     write(rep)
                     continue
             write(delim)
             delim = delimnl
-            self._format(ent, stream, indent,
-                         allowance if last else 1, context, level)
+            self._format(ent, stream, indent, allowance if last else 1, context, level)
 
-    def _pprint_key_val_tuple(self, object, stream, indent, allowance, context,
-                              level):
+    def _pprint_key_val_tuple(self, object, stream, indent, allowance, context, level):
         """Pretty printing for key-value tuples from dict or parameters."""
         k, v = object
         rep = self._repr(k, context, level)
         if isinstance(object, KeyValTupleParam):
             rep = rep.strip("'")
-            middle = '='
+            middle = "="
         else:
-            middle = ': '
+            middle = ": "
         stream.write(rep)
         stream.write(middle)
-        self._format(v, stream, indent + len(rep) + len(middle), allowance,
-                     context, level)
+        self._format(
+            v, stream, indent + len(rep) + len(middle), allowance, context, level
+        )
 
     # Note: need to copy _dispatch to prevent instances of the builtin
     # PrettyPrinter class to call methods of _EstimatorPrettyPrinter (see issue
     # 12906)
-    _dispatch = pprint.PrettyPrinter._dispatch.copy()
+    # mypy error: "Type[PrettyPrinter]" has no attribute "_dispatch"
+    _dispatch = pprint.PrettyPrinter._dispatch.copy()  # type: ignore
     _dispatch[BaseEstimator.__repr__] = _pprint_estimator
     _dispatch[KeyValTuple.__repr__] = _pprint_key_val_tuple
 
@@ -356,9 +379,11 @@ def _safe_repr(object, context, maxlevels, level, changed_only=False):
         items = sorted(object.items(), key=pprint._safe_tuple)
         for k, v in items:
             krepr, kreadable, krecur = saferepr(
-                k, context, maxlevels, level, changed_only=changed_only)
+                k, context, maxlevels, level, changed_only=changed_only
+            )
             vrepr, vreadable, vrecur = saferepr(
-                v, context, maxlevels, level, changed_only=changed_only)
+                v, context, maxlevels, level, changed_only=changed_only
+            )
             append("%s: %s" % (krepr, vrepr))
             readable = readable and kreadable and vreadable
             if krecur or vrecur:
@@ -366,8 +391,9 @@ def _safe_repr(object, context, maxlevels, level, changed_only=False):
         del context[objid]
         return "{%s}" % ", ".join(components), readable, recursive
 
-    if (issubclass(typ, list) and r is list.__repr__) or \
-       (issubclass(typ, tuple) and r is tuple.__repr__):
+    if (issubclass(typ, list) and r is list.__repr__) or (
+        issubclass(typ, tuple) and r is tuple.__repr__
+    ):
         if issubclass(typ, list):
             if not object:
                 return "[]", True, False
@@ -391,7 +417,8 @@ def _safe_repr(object, context, maxlevels, level, changed_only=False):
         level += 1
         for o in object:
             orepr, oreadable, orecur = _safe_repr(
-                o, context, maxlevels, level, changed_only=changed_only)
+                o, context, maxlevels, level, changed_only=changed_only
+            )
             append(orepr)
             if not oreadable:
                 readable = False
@@ -420,16 +447,17 @@ def _safe_repr(object, context, maxlevels, level, changed_only=False):
         items = sorted(params.items(), key=pprint._safe_tuple)
         for k, v in items:
             krepr, kreadable, krecur = saferepr(
-                k, context, maxlevels, level, changed_only=changed_only)
+                k, context, maxlevels, level, changed_only=changed_only
+            )
             vrepr, vreadable, vrecur = saferepr(
-                v, context, maxlevels, level, changed_only=changed_only)
+                v, context, maxlevels, level, changed_only=changed_only
+            )
             append("%s=%s" % (krepr.strip("'"), vrepr))
             readable = readable and kreadable and vreadable
             if krecur or vrecur:
                 recursive = True
         del context[objid]
-        return ("%s(%s)" % (typ.__name__, ", ".join(components)), readable,
-                recursive)
+        return ("%s(%s)" % (typ.__name__, ", ".join(components)), readable, recursive)
 
     rep = repr(object)
-    return rep, (rep and not rep.startswith('<')), False
+    return rep, (rep and not rep.startswith("<")), False
